# Copyright (C) 2001-2003 Nominum, Inc.
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NOMINUM DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

# $Id: rdatatype.py,v 1.2 2003/05/29 00:31:40 halley Exp $

"""DNS Rdata Types."""

import re

NONE = 0
A = 1
NS = 2
MD = 3
MF = 4
CNAME = 5
SOA = 6
MB = 7
MG = 8
MR = 9
NULL = 10
WKS = 11
PTR = 12
HINFO = 13
MINFO = 14
MX = 15
TXT = 16
RP = 17
AFSDB = 18
X25 = 19
ISDN = 20
RT = 21
NSAP = 22
NSAP_PTR = 23
SIG = 24
KEY = 25
PX = 26
GPOS = 27
AAAA = 28
LOC = 29
NXT = 30
SRV = 33
NAPTR = 35
KX = 36
CERT = 37
A6 = 38
DNAME = 39
OPT = 41
UNSPEC = 103
TKEY = 249
TSIG = 250
IXFR = 251
AXFR = 252
MAILB = 253
MAILA = 254
ANY = 255

by_text = {
    'NONE' : NONE,
    'A' : A,
    'NS' : NS,
    'MD' : MD,
    'MF' : MF,
    'CNAME' : CNAME,
    'SOA' : SOA,
    'MB' : MB,
    'MG' : MG,
    'MR' : MR,
    'NULL' : NULL,
    'WKS' : WKS,
    'PTR' : PTR,
    'HINFO' : HINFO,
    'MINFO' : MINFO,
    'MX' : MX,
    'TXT' : TXT,
    'RP' : RP,
    'AFSDB' : AFSDB,
    'X25' : X25,
    'ISDN' : ISDN,
    'RT' : RT,
    'NSAP' : NSAP,
    'NSAP_PTR' : NSAP_PTR,
    'SIG' : SIG,
    'KEY' : KEY,
    'PX' : PX,
    'GPOS' : GPOS,
    'AAAA' : AAAA,
    'LOC' : LOC,
    'NXT' : NXT,
    'SRV' : SRV,
    'NAPTR' : NAPTR,
    'KX' : KX,
    'CERT' : CERT,
    'A6' : A6,
    'DNAME' : DNAME,
    'OPT' : OPT,
    'UNSPEC' : UNSPEC,
    'TKEY' : TKEY,
    'TSIG' : TSIG,
    'IXFR' : IXFR,
    'AXFR' : AXFR,
    'MAILB' : MAILB,
    'MAILA' : MAILA,
    'ANY' : ANY
    }

by_value = {
    NONE : 'NONE',
    A : 'A',
    NS : 'NS',
    MD : 'MD',
    MF : 'MF',
    CNAME : 'CNAME',
    SOA : 'SOA',
    MB : 'MB',
    MG : 'MG',
    MR : 'MR',
    NULL : 'NULL',
    WKS : 'WKS',
    PTR : 'PTR',
    HINFO : 'HINFO',
    MINFO : 'MINFO',
    MX : 'MX',
    TXT : 'TXT',
    RP : 'RP',
    AFSDB : 'AFSDB',
    X25 : 'X25',
    ISDN : 'ISDN',
    RT : 'RT',
    NSAP : 'NSAP',
    NSAP : 'NSAP_PTR_PTR',
    SIG : 'SIG',
    KEY : 'KEY',
    PX : 'PX',
    GPOS : 'GPOS',
    AAAA : 'AAAA',
    LOC : 'LOC',
    NXT : 'NXT',
    SRV : 'SRV',
    NAPTR : 'NAPTR',
    KX : 'KX',
    CERT : 'CERT',
    A6 : 'A6',
    DNAME : 'DNAME',
    OPT : 'OPT',
    UNSPEC : 'UNSPEC',
    TKEY : 'TKEY',
    TSIG : 'TSIG',
    IXFR : 'IXFR',
    AXFR : 'AXFR',
    MAILB : 'MAILB',
    MAILA : 'MAILA',
    ANY : 'ANY'
    }

metatypes = {
    OPT : True
    }

singletons = {
    SOA : True,
    NXT : True,
    DNAME : True,
    # CNAME is technically a singleton, but we allow multiple CNAMEs.
    }

unknown_type_pattern = re.compile('TYPE([0-9]+)$', re.I);

class UnknownRdatatype(Exception):
    """Raised if a type is unknown."""
    pass

def from_text(text):
    """Convert text into a type."""
    value = by_text.get(text.upper())
    if value is None:
        match = unknown_type_pattern.match(text)
        if match == None:
            raise UnknownRdatatype
        value = int(match.group(1))
    return value

def to_text(value):
    """Convert a type into text."""
    if value < 0 or value > 65535:
        raise UnknownRdatatype
    text = by_value.get(value)
    if text is None:
        text = 'TYPE' + `value`
    return text

def is_metatype(rdtype):
    """True if the type is a metatype."""
    if type >= TKEY and type <= ANY or metatypes.has_key(rdtype):
        return True
    return False

def is_singleton(rdtype):
    """True if the type is a singleton."""
    if singletons.has_key(rdtype):
        return True
    return False
